@@ -7,7 +7,7 @@ import os
 import sys
 from contextlib import contextmanager
 from functools import partial
-from typing import TYPE_CHECKING, Any, Dict, FrozenSet, Generator, List, cast
+from typing import TYPE_CHECKING, Any, Dict, FrozenSet, Generator, List, Optional, cast
 
 from kitty.constants import list_kitty_resources
 from kitty.types import run_once
@@ -93,8 +93,8 @@ def launch(args: List[str]) -> None:
     finally:
         sys.stdin = sys.__stdin__
     if result is not None:
-        import json
         import base64
+        import json
         data = base64.b85encode(json.dumps(result).encode('utf-8'))
         sys.stdout.buffer.write(b'\x1bP@kitty-kitten-result|')
         sys.stdout.buffer.write(data)
@@ -122,7 +122,11 @@ def run_kitten(kitten: str, run_name: str = '__main__') -> None:
             print(kitten, file=sys.stderr)
         raise SystemExit(f'No kitten named {original_kitten_name}')
     m = runpy.run_path(path, init_globals={'sys': sys, 'os': os}, run_name='__run_kitten__')
-    m['main'](sys.argv)
+    from kitty.fast_data_types import set_options
+    try:
+        m['main'](sys.argv)
+    finally:
+        set_options(None)
 
 
 @run_once
@@ -151,6 +155,14 @@ def get_kitten_cli_docs(kitten: str) -> Any:
         return ans
 
 
+def get_kitten_wrapper_of(kitten: str) -> str:
+    setattr(sys, 'cli_docs', {})
+    run_kitten(kitten, run_name='__wrapper_of__')
+    ans = getattr(sys, 'cli_docs')
+    delattr(sys, 'cli_docs')
+    return ans.get('wrapper_of') or ''
+
+
 def get_kitten_completer(kitten: str) -> Any:
     run_kitten(kitten, run_name='__completer__')
     ans = getattr(sys, 'kitten_completer', None)
@@ -159,7 +171,7 @@ def get_kitten_completer(kitten: str) -> Any:
     return ans
 
 
-def get_kitten_conf_docs(kitten: str) -> Definition:
+def get_kitten_conf_docs(kitten: str) -> Optional[Definition]:
     setattr(sys, 'options_definition', None)
     run_kitten(kitten, run_name='__conf__')
     ans = getattr(sys, 'options_definition')
@@ -167,6 +179,14 @@ def get_kitten_conf_docs(kitten: str) -> Definition:
     return cast(Definition, ans)
 
 
+def get_kitten_extra_cli_parsers(kitten: str) -> Dict[str,str]:
+    setattr(sys, 'extra_cli_parsers', {})
+    run_kitten(kitten, run_name='__extra_cli_parsers__')
+    ans = getattr(sys, 'extra_cli_parsers')
+    delattr(sys, 'extra_cli_parsers')
+    return cast(Dict[str, str], ans)
+
+
 def main() -> None:
     try:
         args = sys.argv[1:]
@@ -175,4 +195,4 @@ def main() -> None:
         print('Unhandled exception running kitten:')
         import traceback
         traceback.print_exc()
-        input('Press Enter to quit...')
+        input('Press Enter to quit')
