@@ -6,16 +6,21 @@ from collections import deque
 from contextlib import suppress
 from time import monotonic
 from types import TracebackType
-from typing import (
-    TYPE_CHECKING, Any, Callable, ContextManager, Deque, Dict, NamedTuple,
-    Optional, Sequence, Type, Union, cast
-)
+from typing import TYPE_CHECKING, Any, Callable, ContextManager, Deque, Dict, NamedTuple, Optional, Sequence, Type, Union, cast
 
 from kitty.types import DecoratedFunc, ParsedShortcut
 from kitty.typing import (
-    AbstractEventLoop, BossType, Debug, ImageManagerType, KeyActionType,
-    KeyEventType, LoopType, MouseButton, MouseEvent, ScreenSize,
-    TermManagerType
+    AbstractEventLoop,
+    BossType,
+    Debug,
+    ImageManagerType,
+    KeyActionType,
+    KeyEventType,
+    LoopType,
+    MouseButton,
+    MouseEvent,
+    ScreenSize,
+    TermManagerType,
 )
 
 from .operations import MouseTracking, pending_update
@@ -121,11 +126,22 @@ class Handler:
         self._tui_loop.quit(1)
 
     def on_key_event(self, key_event: KeyEventType, in_bracketed_paste: bool = False) -> None:
+        ' Override this method and perform_default_key_action() to handle all key events '
         if key_event.text:
             self.on_text(key_event.text, in_bracketed_paste)
         else:
             self.on_key(key_event)
 
+    def perform_default_key_action(self, key_event: KeyEventType) -> bool:
+        ' Override in sub-class if you want to handle these key events yourself '
+        if key_event.matches('ctrl+c'):
+            self.on_interrupt()
+            return True
+        if key_event.matches('ctrl+d'):
+            self.on_eot()
+            return True
+        return False
+
     def on_text(self, text: str, in_bracketed_paste: bool = False) -> None:
         pass
 
