@@ -11,6 +11,7 @@ from operator import attrgetter
 from time import monotonic
 from typing import (
     Any,
+    Callable,
     Deque,
     Dict,
     Generator,
@@ -28,7 +29,7 @@ from typing import (
 from .borders import Border, Borders
 from .child import Child
 from .cli_stub import CLIOptions
-from .constants import appname, kitten_exe
+from .constants import appname
 from .fast_data_types import (
     GLFW_MOUSE_BUTTON_LEFT,
     GLFW_MOUSE_BUTTON_MIDDLE,
@@ -57,7 +58,7 @@ from .layout.interface import create_layout_object_for, evict_cached_layouts
 from .tab_bar import TabBar, TabBarData
 from .types import ac
 from .typing import EdgeLiteral, SessionTab, SessionType, TypedDict
-from .utils import log_error, platform_window_id, resolved_shell
+from .utils import cmdline_for_hold, log_error, platform_window_id, resolved_shell
 from .window import CwdRequest, Watchers, Window, WindowDict
 from .window_list import WindowList
 
@@ -80,6 +81,7 @@ class TabDict(TypedDict):
     layout_opts: Dict[str, Any]
     enabled_layouts: List[str]
     windows: List[WindowDict]
+    groups: List[Dict[str, Any]]
     active_window_history: List[int]
 
 
@@ -123,6 +125,7 @@ class Tab:  # {{{
     active_bg: Optional[int] = None
     inactive_fg: Optional[int] = None
     inactive_bg: Optional[int] = None
+    confirm_close_window_id: int = 0
 
     def __init__(
         self,
@@ -141,7 +144,7 @@ class Tab:  # {{{
         self.name = getattr(session_tab, 'name', '')
         self.enabled_layouts = [x.lower() for x in getattr(session_tab, 'enabled_layouts', None) or get_options().enabled_layouts]
         self.borders = Borders(self.os_window_id, self.id)
-        self.windows = WindowList(self)
+        self.windows: WindowList = WindowList(self)
         self._last_used_layout: Optional[str] = None
         self._current_layout_name: Optional[str] = None
         self.cwd = self.args.directory
@@ -161,15 +164,24 @@ class Tab:  # {{{
             self._set_current_layout(l0)
             self.startup(session_tab)
 
+    def has_single_window_visible(self) -> bool:
+        if self.current_layout.only_active_window_visible:
+            return True
+        for i, g in enumerate(self.windows.iter_all_layoutable_groups(only_visible=True)):
+            if i > 0:
+                return False
+        return True
+
     def set_enabled_layouts(self, val: Iterable[str]) -> None:
         self.enabled_layouts = [x.lower() for x in val] or ['tall']
         if self.current_layout.name not in self.enabled_layouts:
             self._set_current_layout(self.enabled_layouts[0])
         self.relayout()
 
-    def apply_options(self) -> None:
+    def apply_options(self, is_active: bool) -> None:
+        aw = self.active_window
         for window in self:
-            window.apply_options()
+            window.apply_options(is_active and aw is window)
         self.set_enabled_layouts(get_options().enabled_layouts)
 
     def take_over_from(self, other_tab: 'Tab') -> None:
@@ -287,15 +299,12 @@ class Tab:  # {{{
     def relayout_borders(self) -> None:
         tm = self.tab_manager_ref()
         if tm is not None:
-            w = self.active_window
             ly = self.current_layout
             self.borders(
                 all_windows=self.windows,
                 current_layout=ly, tab_bar_rects=tm.tab_bar_rects,
                 draw_window_borders=(ly.needs_window_borders and self.windows.num_visble_groups > 1) or ly.must_draw_borders
             )
-            if w is not None:
-                w.change_titlebar_color()
 
     def create_layout_object(self, name: str) -> Layout:
         return create_layout_object_for(name, self.os_window_id, self.id)
@@ -474,7 +483,7 @@ class Tab:  # {{{
                                 else:
                                     cmd[:0] = [resolved_shell(get_options())[0]]
                                 cmd[0] = which(cmd[0]) or cmd[0]
-                                cmd[:0] = [kitten_exe(), '__hold_till_enter__']
+                                cmd = cmdline_for_hold(cmd)
         fenv: Dict[str, str] = {}
         if env:
             fenv.update(env)
@@ -772,13 +781,17 @@ class Tab:  # {{{
     def move_window_backward(self) -> None:
         self.move_window(-1)
 
-    def list_windows(self, self_window: Optional[Window] = None) -> Generator[WindowDict, None, None]:
+    def list_windows(self, self_window: Optional[Window] = None, window_filter: Optional[Callable[[Window], bool]] = None) -> Generator[WindowDict, None, None]:
         active_window = self.active_window
         for w in self:
-            yield w.as_dict(
-                is_active=w is active_window,
-                is_focused=w.os_window_id == current_focused_os_window_id() and w is active_window,
-                is_self=w is self_window)
+            if window_filter is None or window_filter(w):
+                yield w.as_dict(
+                    is_active=w is active_window,
+                    is_focused=w.os_window_id == current_focused_os_window_id() and w is active_window,
+                    is_self=w is self_window)
+
+    def list_groups(self) -> List[Dict[str, Any]]:
+        return [g.as_simple_dict() for g in self.windows.groups]
 
     def matches_query(self, field: str, query: str, active_tab_manager: Optional['TabManager'] = None) -> bool:
         if field == 'title':
@@ -848,6 +861,8 @@ class Tab:  # {{{
 
 class TabManager:  # {{{
 
+    confirm_close_window_id: int = 0
+
     def __init__(self, os_window_id: int, args: CLIOptions, wm_class: str, wm_name: str, startup_session: Optional[SessionType] = None):
         self.os_window_id = os_window_id
         self.wm_class = wm_class
@@ -1013,21 +1028,29 @@ class TabManager:  # {{{
     def __len__(self) -> int:
         return len(self.tabs)
 
-    def list_tabs(self, self_window: Optional[Window] = None) -> Generator[TabDict, None, None]:
+    def list_tabs(
+        self, self_window: Optional[Window] = None,
+        tab_filter: Optional[Callable[[Tab], bool]] = None,
+        window_filter: Optional[Callable[[Window], bool]] = None
+    ) -> Generator[TabDict, None, None]:
         active_tab = self.active_tab
         for tab in self:
-            yield {
-                'id': tab.id,
-                'is_focused': tab is active_tab and tab.os_window_id == current_focused_os_window_id(),
-                'is_active': tab is active_tab,
-                'title': tab.name or tab.title,
-                'layout': str(tab.current_layout.name),
-                'layout_state': tab.current_layout.layout_state(),
-                'layout_opts': tab.current_layout.layout_opts.serialized(),
-                'enabled_layouts': tab.enabled_layouts,
-                'windows': list(tab.list_windows(self_window)),
-                'active_window_history': list(tab.windows.active_window_history),
-            }
+            if tab_filter is None or tab_filter(tab):
+                windows = list(tab.list_windows(self_window, window_filter))
+                if windows:
+                    yield {
+                        'id': tab.id,
+                        'is_focused': tab is active_tab and tab.os_window_id == current_focused_os_window_id(),
+                        'is_active': tab is active_tab,
+                        'title': tab.name or tab.title,
+                        'layout': str(tab.current_layout.name),
+                        'layout_state': tab.current_layout.layout_state(),
+                        'layout_opts': tab.current_layout.layout_opts.serialized(),
+                        'enabled_layouts': tab.enabled_layouts,
+                        'windows': windows,
+                        'groups': tab.list_groups(),
+                        'active_window_history': list(tab.windows.active_window_history),
+                    }
 
     def serialize_state(self) -> Dict[str, Any]:
         return {
@@ -1218,8 +1241,9 @@ class TabManager:  # {{{
         del self.tabs
 
     def apply_options(self) -> None:
+        at = self.active_tab
         for tab in self:
-            tab.apply_options()
+            tab.apply_options(at is tab)
         self.tab_bar_hidden = get_options().tab_bar_style == 'hidden'
         self.tab_bar.apply_options()
         self.update_tab_bar_data()
